@@ -23,11 +23,11 @@ Test::Which - Skip tests if external programs are missing from PATH (with versio
 
 =head1 VERSION
 
-Version 0.05
+Version 0.06
 
 =cut
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 =head1 SYNOPSIS
 
@@ -360,107 +360,132 @@ sub which_ok {
 sub _capture_version_output {
 	my ($path, $custom_flags) = @_;
 
-	# Return undef immediately if path is not defined
 	return undef unless defined $path;
 
-	# Create cache key from path and flags
+	# Build cache key
 	my $cache_key = $path;
 	if (defined $custom_flags) {
-		if (ref($custom_flags) eq 'ARRAY') {
+		if (ref $custom_flags eq 'ARRAY') {
 			$cache_key .= '|' . join(',', @$custom_flags);
-		} elsif (!ref($custom_flags)) {
+		} elsif (!ref $custom_flags) {
 			$cache_key .= '|' . $custom_flags;
 		}
 	}
-
-	# Return cached result if available
 	return $VERSION_CACHE{$cache_key} if exists $VERSION_CACHE{$cache_key};
 
-	# Determine which flags to try
+	# Determine flags to try
 	my @flags;
-	if (defined $custom_flags) {
-		if (ref($custom_flags) eq 'ARRAY') {
-			@flags = @$custom_flags;
-		} elsif (!ref($custom_flags)) {
-			@flags = ($custom_flags);
-		} else {
-			warn 'Invalid version_flag type: ', ref($custom_flags);
-			$VERSION_CACHE{$cache_key} = undef;
-			return undef;
-		}
-	} else {
+	if (!defined $custom_flags) {
 		@flags = qw(--version -version -v -V);
-		# Add Windows-specific flags
 		push @flags, qw(/? -?) if $^O eq 'MSWin32';
 	}
-
-	my $timeout = $TIMEOUT;
-
-	for my $flag (@flags) {
-		# Build argv-style command: avoid shell when possible
-		my @cmd = ($path);
-		push @cmd, $flag if defined $flag && length $flag;
-
-		my $out;
-		my $ok;
-
-		if (eval { require IPC::Run3; 1 }) {
-			# Prefer IPC::Run3 to avoid shell and capture stderr
-			eval {
-				local $SIG{ALRM} = sub { die 'TIMEOUT' };
-				alarm $timeout;
-				my ($in, $stdout, $stderr) = ('', '', '');
-				IPC::Run3::run3(\@cmd, \$in, \$stdout, \$stderr);
-				alarm 0;
-				$out = $stdout . $stderr;
-			};
-			if ($@) {
-				if ($@ !~ /TIMEOUT/) {
-					warn "Unexpected probe error while probing: $@";
-				}
-				next;
-			}
-			$ok = defined $out && $out ne '';
-		} else {
-			# Fallback: build a safe shell command (best-effort) and qx it with alarm
-			my $shell_cmd;
-			if ($^O eq 'MSWin32') {
-				# Windows: quote via double quotes
-				my $flagpart = defined $flag ? " $flag" : '';
-				$shell_cmd = qq{"$path"$flagpart 2>&1};
-			} else {
-				# Unix: single-quote path (escape single quotes inside)
-				my $escaped = $path;
-				$escaped =~ s/'/'\\''/g;
-				my $flagpart = '';
-				if (defined $flag && length $flag) {
-					# flags containing spaces should be used carefully
-					$flagpart = ' ' . $flag;
-				}
-				$shell_cmd = qq{'$escaped'$flagpart 2>&1};
-			}
-
-			eval {
-				local $SIG{ALRM} = sub { die 'TIMEOUT' };
-				alarm $timeout;
-				$out = qx{$shell_cmd};
-				alarm 0;
-			};
-			if ($@) {
-				next if $@ =~ /TIMEOUT/;
-				next;
-			}
-			$ok = defined $out && $out ne '';
-		}
-
-		next unless $ok;
-
-		# Cache and return the result
-		$VERSION_CACHE{$cache_key} = $out;
-		return $out;
+	elsif (ref($custom_flags) eq 'ARRAY') {
+		@flags = @$custom_flags;
+	}
+	elsif (!ref($custom_flags)) {
+		@flags = ($custom_flags);	# allow empty string ''
+	}
+	else {
+		warn "Invalid version_flag type: ", ref($custom_flags);
+		$VERSION_CACHE{$cache_key} = undef;
+		return undef;
 	}
 
-	# Cache the failure (undef) so we don't keep retrying
+	# timeout (default to 5 seconds if not set)
+	my $timeout = defined $TIMEOUT ? $TIMEOUT : 5;
+
+	my $is_win = ($^O eq 'MSWin32') ? 1 : 0;
+	my $is_bat = ($path =~ /\.(bat|cmd)$/i) ? 1 : 0;
+
+	FLAG:
+	for my $flag (@flags) {
+
+		# Build command / args
+		my @cmd;
+		if ($is_win && $is_bat) {
+			# For .bat/.cmd on Windows, call cmd.exe /c "prog [flag]"
+			# Build a single command string for cmd.exe /c; quote path if it contains spaces
+			my $path_part = ($path =~ /\s/) ? qq{"$path"} : $path;
+			my $cmdstr = $path_part;
+			$cmdstr .= " $flag" if defined $flag && length $flag;
+			@cmd = ('cmd.exe', '/c', $cmdstr);
+		} else {
+			# Normal argv-style call for binaries / scripts
+			@cmd = ($path);
+			push @cmd, $flag if defined $flag && length $flag;
+		}
+
+		my ($stdout, $stderr) = ('', '');
+		my $ok = 0;
+
+		# Try IPC::Run3 (preferred) in argv-mode
+		if (eval { require IPC::Run3; 1 }) {
+			eval {
+				local $SIG{ALRM} = sub { die 'TIMEOUT' };
+				alarm $timeout;
+				IPC::Run3::run3(\@cmd, \undef, \$stdout, \$stderr);
+				alarm 0;
+			};
+			if ($@) {
+				next FLAG if $@ =~ /TIMEOUT/;
+				next FLAG;
+			}
+			$ok = ($stdout ne '' || $stderr ne '');
+		}
+
+		# Fallback to shell qx{} if IPC::Run3 not available or produced no output
+		if (!$ok) {
+			my $shell_cmd;
+			if ($is_win && $is_bat) {
+				# Use cmd.exe /c "prog [flag]" and capture stderr
+				my $path_part = ($path =~ /\s/) ? qq{"$path"} : $path;
+				my $inner = $path_part;
+				$inner .= " $flag" if defined $flag && length $flag;
+				$shell_cmd = qq{cmd.exe /c "$inner" 2>&1};
+			}
+			elsif ($is_win) {
+				# Non-bat on Windows — quote path and append flag
+				my $flagpart = defined $flag && length $flag ? " $flag" : '';
+				$shell_cmd = qq{"$path"$flagpart 2>&1};
+			}
+			else {
+				# Unix: single-quote the path; if flag present pass it unquoted (shell will split)
+				my $escaped = $path;
+				$escaped =~ s/'/'\\''/g;
+				if (defined $flag && length $flag) {
+					# If flag contains spaces, shell will treat it as one word if quoted; use simple approach
+					my $f = $flag;
+					$f =~ s/'/'\\''/g;
+					$shell_cmd = qq{'$escaped' '$f' 2>&1};
+				} else {
+					$shell_cmd = qq{'$escaped' 2>&1};
+				}
+			}
+
+			eval {
+				local $SIG{ALRM} = sub { die 'TIMEOUT' };
+				alarm $timeout;
+				$stdout = qx{$shell_cmd};
+				alarm 0;
+			};
+			next FLAG if $@;
+			$ok = ($stdout ne '');
+		}
+
+		next FLAG unless $ok;
+
+		# Merge outputs (IPC::Run3 already gave us stderr separately)
+		my $output = $stdout . $stderr;
+
+		# Normalize newlines on Windows
+		$output =~ s/\r\n/\n/g if $is_win;
+
+		# Cache and return
+		$VERSION_CACHE{$cache_key} = $output;
+		return $output;
+	}
+
+	# Nothing worked — cache failure
 	$VERSION_CACHE{$cache_key} = undef;
 	return undef;
 }
@@ -841,7 +866,9 @@ L<Test::Builder> - Used for test integration
 =over 4
 
 =item * Version detection is heuristic-based and may fail for programs with
-unusual output formats. Use custom C<version_flag> or C<extractor> for such cases.
+unusual output formats.
+Use custom C<version_flag> or C<extractor> for such cases,
+though even those may break.
 
 =item * No built-in timeout for program execution. Hanging programs will hang tests.
 
